@@ -52,7 +52,7 @@ void rocrand_init_kernel(GeneratorState*    states,
 
 template<class GeneratorState>
 __global__
-void rocrand_kernel(unsigned int* output, const size_t size)
+void rocrand_kernel(unsigned long long* output, const size_t size)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -191,23 +191,23 @@ void rocrand_discrete_kernel(unsigned int*                 output,
     }
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand_state_threefry_type)
+TEST(rocrand_kernel_threefry2x64_20, rocrand_state_threefry_type)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
-    EXPECT_EQ(alignof(state_type), alignof(uint2));
+    typedef rocrand_state_threefry2x64_20 state_type;
+    EXPECT_EQ(alignof(state_type), alignof(ulonglong2));
     EXPECT_EQ(sizeof(state_type[32]), 32 * sizeof(state_type));
-    // TODO: Enable once uint2 trivially copyable.
+    // TODO: Enable once ulonglong2 trivially copyable.
     //EXPECT_TRUE(std::is_trivially_copyable<state_type>::value);
     EXPECT_TRUE(std::is_trivially_destructible<state_type>::value);
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand)
+TEST(rocrand_kernel_threefry2x64_20, rocrand)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
-    const size_t  output_size = 8192;
-    unsigned int* output;
-    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
+    const size_t        output_size = 8192;
+    unsigned long long* output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned long long)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_kernel<state_type>),
@@ -219,10 +219,10 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand)
                        output_size);
     HIP_CHECK(hipGetLastError());
 
-    std::vector<unsigned int> output_host(output_size);
+    std::vector<unsigned long long> output_host(output_size);
     HIP_CHECK(hipMemcpy(output_host.data(),
                         output,
-                        output_size * sizeof(unsigned int),
+                        output_size * sizeof(unsigned long long),
                         hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
@@ -230,15 +230,15 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand)
     double mean = 0;
     for(auto v : output_host)
     {
-        mean += static_cast<double>(v) / UINT_MAX;
+        mean += static_cast<double>(v) / static_cast<double>(ULLONG_MAX);
     }
     mean = mean / output_size;
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform)
+TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
@@ -269,9 +269,9 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform)
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_double)
+TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_double)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const size_t output_size = 8192;
     double*      output;
@@ -302,9 +302,9 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_double)
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_range)
+TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_range)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const size_t output_size = 1 << 26;
     float*       output;
@@ -333,9 +333,9 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_range)
     }
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_double_range)
+TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_double_range)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const size_t output_size = 1 << 26;
     double*      output;
@@ -364,9 +364,9 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_double_range)
     }
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand_normal)
+TEST(rocrand_kernel_threefry2x64_20, rocrand_normal)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
@@ -405,9 +405,9 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand_normal)
     EXPECT_NEAR(stddev, 1.0, 0.2);
 }
 
-TEST(rocrand_kernel_threefry2x32_20, rocrand_log_normal)
+TEST(rocrand_kernel_threefry2x64_20, rocrand_log_normal)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
@@ -450,12 +450,12 @@ TEST(rocrand_kernel_threefry2x32_20, rocrand_log_normal)
     EXPECT_NEAR(0.25, logstd, 0.25 * 0.2);
 }
 
-class rocrand_kernel_threefry2x32_20_poisson : public ::testing::TestWithParam<double>
+class rocrand_kernel_threefry2x64_20_poisson : public ::testing::TestWithParam<double>
 {};
 
-TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_poisson)
+TEST_P(rocrand_kernel_threefry2x64_20_poisson, rocrand_poisson)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const double lambda = GetParam();
 
@@ -466,7 +466,7 @@ TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_poisson)
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_poisson_kernel<state_type>),
                        dim3(4),
-                       dim3(64),
+                       dim3(32),
                        0,
                        0,
                        output,
@@ -500,9 +500,9 @@ TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_poisson)
     EXPECT_NEAR(variance, lambda, std::max(1.0, lambda * 1e-1));
 }
 
-TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_discrete)
+TEST_P(rocrand_kernel_threefry2x64_20_poisson, rocrand_discrete)
 {
-    typedef rocrand_state_threefry2x32_20 state_type;
+    typedef rocrand_state_threefry2x64_20 state_type;
 
     const double lambda = GetParam();
 
@@ -516,7 +516,7 @@ TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_discrete)
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_discrete_kernel<state_type>),
                        dim3(4),
-                       dim3(64),
+                       dim3(32),
                        0,
                        0,
                        output,
@@ -553,6 +553,6 @@ TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_discrete)
 
 const double lambdas[] = {1.0, 5.5, 20.0, 100.0, 1234.5, 5000.0};
 
-INSTANTIATE_TEST_SUITE_P(rocrand_kernel_threefry2x32_20_poisson,
-                         rocrand_kernel_threefry2x32_20_poisson,
+INSTANTIATE_TEST_SUITE_P(rocrand_kernel_threefry2x64_20_poisson,
+                         rocrand_kernel_threefry2x64_20_poisson,
                          ::testing::ValuesIn(lambdas));
